# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-19 22:55
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0005_auto_20170614_1607'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dancetypelevel',
            name='requiresAudition',
        ),
    ]
