/*!
 * Start Bootstrap - Busines Frontpage (https://startbootstrap.com/template-overviews/business-frontpage)
 * Copyright 2013-2017 Start Bootstrap
 * Licensed under MIT (https://github.com/BlackrockDigital/startbootstrap-business-frontpage/blob/master/LICENSE)
 */

body {
  padding-top: 54px;
}

@media (min-width: 992px) {
  body {
    padding-top: 56px;
  }

  /* Offset body positioning to ensure that navbar stays at the top */
  .navbar-top {
    position: relative;
    top: -56px;
  }

  .splash-header {
    top: -56px;
  }

}

/* Offset body positioning to ensure that navbar stays at the top */
.navbar-top {
  position: relative;
  top: -54px;
}

.splash-header {
  height: 50vh;
  min-height: 300px;
  -webkit-background-size: cover;
  -moz-background-size: cover;
  background-size: cover;
  -o-background-size: cover;
  position: relative;
  top: -54px;
}

.card {
  height: 100%;
}

.footer {
  color: #fff;
}
