# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-05-12 14:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='instructorlistpluginmodel',
            name='title',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Listing Title'),
        ),
        migrations.AlterField(
            model_name='instructorlistpluginmodel',
            name='template',
            field=models.CharField(blank=True, max_length=250, null=True, verbose_name='Template'),
        ),
        migrations.AlterField(
            model_name='locationlistpluginmodel',
            name='template',
            field=models.CharField(blank=True, max_length=250, null=True, verbose_name='Template'),
        ),
        migrations.AlterField(
            model_name='locationpluginmodel',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.Location', verbose_name='Location'),
        ),
        migrations.AlterField(
            model_name='locationpluginmodel',
            name='template',
            field=models.CharField(blank=True, max_length=250, null=True, verbose_name='Template'),
        ),
    ]
