{% load thumbnail %}

<header class="splash-header" style="background: url('{% spaceless %}
    {% if instance.external_picture %}
        {{ instance.external_picture }}
    {% elif instance.use_no_cropping %}
        {{ instance.picture.url }}
    {% elif instance.picture %}
        {% thumbnail instance.picture picture_size.size crop=picture_size.crop upscale=picture_size.upscale subject_location=instance.picture.subject_location as thumbnail %}
        {{ thumbnail.url }}
    {% endif %}
{% endspaceless %}') top center no-repeat scroll;">
