# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-17 16:42
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('faq', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='faq',
            options={'ordering': ('orderNum',), 'verbose_name': 'FAQ', 'verbose_name_plural': 'FAQs'},
        ),
        migrations.AlterModelOptions(
            name='faqcategory',
            options={'ordering': ('orderNum',), 'verbose_name': 'FAQ category', 'verbose_name_plural': 'FAQ categories'},
        ),
        migrations.AlterField(
            model_name='faq',
            name='answer',
            field=djangocms_text_ckeditor.fields.HTMLField(help_text='Answer the question.', verbose_name='Answer'),
        ),
        migrations.AlterField(
            model_name='faq',
            name='category',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='faq.FAQCategory', verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='faq',
            name='creationDate',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Creation date'),
        ),
        migrations.AlterField(
            model_name='faq',
            name='draft',
            field=models.BooleanField(default=False, help_text='Check this box to prevent publication.', verbose_name='Draft status'),
        ),
        migrations.AlterField(
            model_name='faq',
            name='modifiedDate',
            field=models.DateTimeField(auto_now=True, verbose_name='Last modified date'),
        ),
        migrations.AlterField(
            model_name='faq',
            name='orderNum',
            field=models.PositiveIntegerField(default=0, help_text='This number specifies the order in which the questions will be shown on the FAQ page.', verbose_name='Order number'),
        ),
        migrations.AlterField(
            model_name='faq',
            name='question',
            field=models.CharField(max_length=200, verbose_name='Question'),
        ),
        migrations.AlterField(
            model_name='faqcategory',
            name='name',
            field=models.CharField(max_length=200, unique=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='faqcategory',
            name='orderNum',
            field=models.PositiveIntegerField(default=0, help_text='This number specifies the order in which categories will be shown.', verbose_name='Order number'),
        ),
        migrations.AlterField(
            model_name='faqcategorypluginmodel',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='faq.FAQCategory', verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='faqsinglepluginmodel',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='faq.FAQ', verbose_name='Question'),
        ),
    ]
