# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-08-09 02:47
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0020_auto_20180808_2247'),
        ('vouchers', '0003_auto_20170724_2103'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomerGroupVoucher',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.CustomerGroup', verbose_name='Customer group')),
                ('voucher', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='vouchers.Voucher', verbose_name='Voucher')),
            ],
            options={
                'verbose_name': 'Group-specific voucher restriction',
                'verbose_name_plural': 'Group-specific voucher restrictions',
            },
        ),
    ]
