{% extends "cms/site_base.html" %}
{% load static sekizai_tags cms_tags %}

{% block extra_head %}

	{% addtoblock "meta" %}
		<meta name="robots" content="index, follow">
	{% endaddtoblock %}

	{# Fitvids code (to ensure Youtube videos always fit) #}
	{% addtoblock "js" %}
	<script type="text/javascript" src="{% static 'fitvids/jquery.fitvids.js' %}"></script>
	{% endaddtoblock %}
	{% addtoblock "js" %}
	<script type="text/javascript">
	    $(".content-col").fitVids();
	</script>
	{% endaddtoblock %}

{% endblock %}

{% block footer %}{% static_placeholder "footer" %}{% endblock %}
