{% load i18n %}
{% blocktrans %}Dear {{ name }},{% endblocktrans %}

{% trans "This is an automatically generated email is to remind you of the following event/task" %}:

{{ event.name }}

{% if occurrence.allDay %}{{ occurrence.startTime|date:"D., F j, Y" }}{% if occurrence.startTime.date != occurrence.endTime.date %} {% trans "to" %} {{ occurrence.endTime|date:"D., F j, Y" }}{% endif %}{% else %}{{ occurrence.startTime|date:"D., F j, Y, P" }}{% if occurrence.startTime.date != occurrence.endTime.date %} {% trans "to" %} {{ occurrence.endTime|date:"D., F j, Y P" }}{% else %} - {{ occurrence.endTime|date:"P" }}{% endif %}{% endif %}

{% if event.description %}{% trans "Description" %}: {{ event.description }}
{% endif %}{% if event.location %}{% trans "Location" %}: {{ event.location }}
{% endif %}{% if event.category %}{% trans "Category" %}: {{ event.category }}{% endif %}

{% trans "Thanks!" %}
{{ businessName }}