# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-17 16:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paypal', '0002_auto_20170614_1607'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='paymentrecord',
            options={'verbose_name': 'Paypal payment record', 'verbose_name_plural': 'Payment records'},
        ),
    ]
