{% extends "cms/admin_home.html" %}
{% load cms_tags i18n %}

{% block content %}

<h2 class="mt-4">{% trans "School Performance Statistics" %}</h2>

<div class="card card-body">
  <table class="classregsummary table mb-0">
  <tr>
      <td>{% trans "Total # Students" %}:</td><td> {{ totalStudents }}</td>
  </tr>
  <tr>
    <td>{% trans "Total # Series Taught" %}:</td><td> {{ numSeries }} </td>
  </tr>
  <tr>
    <td>{% trans "Total # Series Registrations" %}: </td><td> {{ totalSeriesRegs }} </td>
  </tr>
  <tr>
      <td>{% trans "Total Time Studio in Existence" %}:</td> <td>{{ totalTime }}</td>
  </tr>
  </table>
</div>

{% static_placeholder 'stats_graphs' %}

<hr />

{% endblock %}