# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-17 16:42
from __future__ import unicode_literals

import datetime
from django.db import migrations, models
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='newsitem',
            options={'ordering': ('-pinThis', '-publicationDate'), 'verbose_name': 'News item', 'verbose_name_plural': 'News items'},
        ),
        migrations.AlterField(
            model_name='latestnewspluginmodel',
            name='template',
            field=models.CharField(blank=True, max_length=250, null=True, verbose_name='Template'),
        ),
        migrations.AlterField(
            model_name='newsitem',
            name='alert',
            field=models.BooleanField(default=False, help_text='Alerts (such as cancellations) may be displayed in an emphasized fashion.', verbose_name='Alert'),
        ),
        migrations.AlterField(
            model_name='newsitem',
            name='content',
            field=djangocms_text_ckeditor.fields.HTMLField(help_text='Insert news content here.', verbose_name='Content'),
        ),
        migrations.AlterField(
            model_name='newsitem',
            name='creationDate',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Creation date'),
        ),
        migrations.AlterField(
            model_name='newsitem',
            name='draft',
            field=models.BooleanField(default=False, help_text='Check to hide from publication', verbose_name='Draft status'),
        ),
        migrations.AlterField(
            model_name='newsitem',
            name='modifiedDate',
            field=models.DateTimeField(auto_now=True, help_text='Time of most recent edit', verbose_name='Last modified date'),
        ),
        migrations.AlterField(
            model_name='newsitem',
            name='publicationDate',
            field=models.DateTimeField(default=datetime.datetime.now, verbose_name='Publication date'),
        ),
        migrations.AlterField(
            model_name='newsitem',
            name='title',
            field=models.CharField(help_text='Title the news item. Make sure this is descriptive.', max_length=200, verbose_name='Title'),
        ),
    ]
