# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-05-12 22:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='StatsGraphPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='stats_statsgraphpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(blank=True, max_length=250, null=True, verbose_name='Template')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]
