# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-21 10:53
from __future__ import unicode_literals

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('core', '0011_auto_20170720_1945'),
    ]

    operations = [
        migrations.CreateModel(
            name='StripeCharge',
            fields=[
                ('paymentrecord_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.PaymentRecord')),
                ('chargeId', models.CharField(max_length=32, verbose_name='Charge Id')),
                ('status', models.CharField(blank=True, max_length=30, null=True, verbose_name='Current status')),
            ],
            options={
                'verbose_name_plural': 'Stripe charges',
                'verbose_name': 'Stripe charge',
            },
            bases=('core.paymentrecord',),
        ),
        migrations.CreateModel(
            name='StripeChargeFormModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='stripe_stripechargeformmodel', serialize=False, to='cms.CMSPlugin')),
                ('defaultAmount', models.FloatField(default=0, help_text='The initial value for gift certificate forms.', verbose_name='Default amount')),
                ('successPage', cms.models.fields.PageField(help_text='When the user returns to the site after a successful transaction, send them to this page.', on_delete=django.db.models.deletion.CASCADE, related_name='successPageForStripe', to='cms.Page', verbose_name='Success Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]
