{% extends "cms/home.html" %}
{% load cms_tags danceschool_tags %}

{% block content_header %}
    <!-- Header with Background Image -->
    {% placeholder splash_image or %}
    <header class="splash-header" style="background: url('https://via.placeholder.com/1920x500?text=+') top center no-repeat scroll;">
    {% endplaceholder %}
        <div class="container">
          <div class="row">
            <div class="col-lg-12">
              <h1 class="display-3 text-center text-white mt-4">{% placeholder "splash_title" or %}{% getSchoolName %}{% endplaceholder %}</h1>
              {% placeholder 'splash_caption' %}
            </div>
          </div>
        </div>
    </header>
{% endblock %}