# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-06-14 16:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0005_auto_20170614_1607'),
        ('paypal', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('paymentId', models.CharField(max_length=50, unique=True, verbose_name='Paypal Payment ID')),
                ('payerId', models.CharField(blank=True, max_length=50, null=True, verbose_name='Paypal Payer ID')),
                ('status', models.CharField(blank=True, max_length=30, null=True, verbose_name='Current status')),
                ('creationDate', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('modifiedDate', models.DateTimeField(auto_now=True, verbose_name='Last updated')),
            ],
        ),
        migrations.RemoveField(
            model_name='ipncartitem',
            name='ipn',
        ),
        migrations.RemoveField(
            model_name='ipncartitem',
            name='revenueItem',
        ),
        migrations.RemoveField(
            model_name='ipnmessage',
            name='finalRegistration',
        ),
        migrations.RemoveField(
            model_name='ipnmessage',
            name='paypalInvoice',
        ),
        migrations.RemoveField(
            model_name='ipnmessage',
            name='priorTransaction',
        ),
        migrations.RemoveField(
            model_name='ipnmessage',
            name='registration',
        ),
        migrations.RemoveField(
            model_name='paynowformmodel',
            name='cancellationPage',
        ),
        migrations.DeleteModel(
            name='Invoice',
        ),
        migrations.DeleteModel(
            name='IPNCartItem',
        ),
        migrations.DeleteModel(
            name='IPNMessage',
        ),
        migrations.AddField(
            model_name='paymentrecord',
            name='invoice',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='paypalpayments', to='core.Invoice', verbose_name='Invoice'),
        ),
    ]
